#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <iostream>

#include "mystd.h"
#include "olm.h"
#include "asi.h"
#include "r-temp.h"

using namespace std;

namespace NR_Temp{
    const int NDET = 19;
    const int NREC_HISTO = 50000;
}

void NR_Temp::AnalyseDopp( ) 
{
    string fname = sask( "analyse data from file (.dopp)" );
    FILE *fd;
    string inFile = fname + ".dopp";
    if (!(fd = fopen(inFile.c_str(), "r"))) {
        printf ("! cannot open file %s\n", inFile.c_str());
        return;
    }
    static int maxtim = 0;
    maxtim = iask( "maximum number of points to read (0: read all)", maxtim );
    int ipos, iold, ibaddiff;
    int itim = 0;
    while( (maxtim==0 || itim<maxtim) && fscanf( fd, "%i\n", &ipos )==1 ){
        ibaddiff = (ipos | 1) - (iold | 1);
        if( itim>0 && abs(ipos-iold)>4 ){
            printf( "%6i  %5i %5i %5i\n", itim, ipos, ipos-iold, ibaddiff );
        }
        iold = ipos;
        ++itim;
    }
    fclose( fd );
}

void NR_Temp::AnalyseChop( ) 
{
    string fname = sask( "analyse data from file (.chop)" );
    FILE *fd;
    string inFile = fname + ".chop";
    if (!(fd = fopen(inFile.c_str(), "r"))) {
        printf ("! cannot open file %s\n", inFile.c_str());
        return;
    }
    static int maxtim = 0;
    maxtim = iask( "maximum number of points to read (0: read all)", maxtim );
    int ipos, iold;
    int itim = 0;
    while( (maxtim==0 || itim<maxtim) && fscanf( fd, "%i\n", &ipos )==1 ){
        if( itim>0 && abs(ipos-iold)>1 ){
            printf( "%6i  %5i %5i\n", itim, ipos, ipos-iold );
        }
        iold = ipos;
        ++itim;
    }
    fclose( fd );
}

void NR_Temp::ReadHisto(void)
{
    COld olf;
        
    string inFile = wask("Read data from histogram file");

    FILE *fd;
    if (!(fd = fopen(inFile.c_str(), "r"))) {
        printf ("! cannot open file %s\n", inFile.c_str());
        return;
    }

    mystd::fname_divide( inFile, 0, &(olf.name), 0 );
    olf.lDoc.push_back( "data from histogram "+inFile );

    olf.xco = CCoord( "t", "" );
    olf.yco = CCoord( "cts", "" );
    olf.ZCo.push_back( CCoord( "det", "") );

    const int nz0 = NDET; // Detektoren
    const int ni  = NREC_HISTO;
    
    CScan S;
    unsigned int cnt;
    for ( int j0=0; j0<nz0; ++j0 ){
        S.Clear();
        S.z.resize( 1 );
        S.z[0] = j0 ;
        for( int i=0; i<ni; ++i ){
            if( fread( &cnt, 4, 1, fd )!=1 ){
                printf( "failed reading detector %i channel %i\n", j0, i );
                break;
            }
            S.push_back( (double)i, (double)cnt );
        }
        olf.VS.push_back(S);
    }

    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}


void NR_Temp::ReadAcq0( bool diff, bool per_direction )
{
    COld olf;
        
    string file_f, file_i;
    file_f = wask("Read data from acquire-v0 file");
    if( diff) file_i = wask("Subtract data up to and including file");

    FILE *fd_f, *fd_i;
    if (!(fd_f = fopen(file_f.c_str(), "r"))) {
        printf ("! cannot open file %s\n", file_f.c_str());
        return;
    }
    if ( diff && !(fd_i = fopen(file_i.c_str(), "r"))) {
        printf ("! cannot open file %s\n", file_i.c_str());
        return;
    }

    mystd::fname_divide( file_f, 0, &(olf.name), 0 );
    olf.lDoc.push_back( "acquire data from "+file_f );
    if( diff ) olf.lDoc.push_back( ".. subtracted data up to "+file_i );

    olf.xco = CCoord( "E", "ueV" );
    olf.yco = CCoord( "cts", "sec-1" );
    olf.ZCo.push_back( CCoord( "det", "") );
    if( per_direction )
        olf.ZCo.push_back( CCoord( "way", "") );

    const int ndet = NDET; // Detektoren
    
    CScan S;
    for( int j0=0; j0<ndet; ++j0 ){
        if( per_direction ){
            for( int j1=0; j1<2; ++j1 ){
                S.Clear();
                S.z.resize( 2 );
                S.z[0] = j0 ;
                S.z[1] = j1 ;
                olf.VS.push_back(S);
            }
        } else {
            S.Clear();
            S.z.resize( 1 );
            S.z[0] = j0 ;
            olf.VS.push_back(S);
        }
    }

    string lin;
    vector<double> dat_f, dat_i;
    int j1 = 0, nline=0;
    double x, y, tmeas, ttot=0;
    while( mystd::freadln(fd_f, &lin)>0 ){
        if( mystd::str2vec(lin.substr(1), &dat_f, 0, false) ) break;
        if( dat_f.size()!=ndet+2 ){
            cout << "! line "<<nline<<" has size "<<dat_f.size()<<
                " instead of "<<ndet+2<<"\n";
            return;
        }
        if( diff ){
            if( mystd::freadln(fd_i, &lin) <= 0 ){
                cout << "! incongruent initial file length\n";
                return;
            }
            if( mystd::str2vec(lin.substr(1), &dat_i, 0, false) ){
                cout << "! incongruent line in initial file\n";
                return;
            }
            if( dat_i.size()!=ndet+2 ){
                cout << "! initial file: line "<<nline<<" has size "
                     <<dat_i.size()<<" instead of "<<ndet+2<<"\n";
                return;
            }
        }                
        x = dat_f[0];
        if( per_direction || j1==0 )
            tmeas = 0;
        tmeas += dat_f[ndet+1]; // number of 20usec frames
        if( diff ){
            if( dat_i[0] != x ){
                cout<<"! inconsistent energy\n";
                return;
            }
            tmeas -= dat_i[ndet+1];
            if( tmeas<0 ){
                cout<<"! negative measuring time\n";
                return;
            }
        }
        // cout<<"measured "<<tmeas<<" times at "<<x<<"\n";
        if( tmeas<1 ) continue;
        for ( int j0=0; j0<ndet; ++j0 ){
            y = dat_f[1+j0];
            if( diff ) y -= dat_i[1+j0];
            y = y;
            if( per_direction ){
                olf.VS[j0*2+j1].push_back( x, y );
            } else {
                if( j1==0 ){
                    olf.VS[j0].push_back( x, y );
                } else {
                    olf.VS[j0].y[ olf.VS[j0].size()-1 ] += y;
                }
            }
        }
        // convert to counts per second
        if( per_direction || j1==1 ){
            ttot += tmeas*2e-5;
            for ( int j0=0; j0<ndet; ++j0 ){
                olf.VS[j0].y[ olf.VS[j0].size()-1 ] = 
                    olf.VS[j0].y[ olf.VS[j0].size()-1 ] / tmeas / 2e-5;
            }
        }
        j1 = (++j1)%2;
        ++nline;
    }

    cout<<"effective acquisition time "<<ttot<<" sec\n";
    olf.RPar.push_back( CParam( "eff. acquisition time", "sec", ttot ) );
    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}


void NR_Temp::ReadCho0( bool diff )
{
    COld olf;
        
    string file_f, file_i;
    file_f = wask("Read chopper histogram file");
    if( diff) file_i = wask("Subtract data up to and including file");

    FILE *fd_f, *fd_i;
    if (!(fd_f = fopen(file_f.c_str(), "r"))) {
        printf ("! cannot open file %s\n", file_f.c_str());
        return;
    }
    if ( diff && !(fd_i = fopen(file_i.c_str(), "r"))) {
        printf ("! cannot open file %s\n", file_i.c_str());
        return;
    }

    mystd::fname_divide( file_f, 0, &(olf.name), 0 );
    olf.lDoc.push_back( "chopper histogram from "+file_f );
    if( diff ) olf.lDoc.push_back( ".. subtracted data up to "+file_i );

    olf.xco = CCoord( "phase", "" );
    olf.yco = CCoord( "cts", "sec-1" );
    olf.ZCo.push_back( CCoord( "det", "" ) );
    olf.ZCo.push_back( CCoord( "E-range", "" ) );

    const int ndet = NDET; // Detektoren
    static int ng = 2; // was 4 in some very old measurements
    // ng = iask( "number of energy groups", ng );
    
    CScan S;
    for( int j0=0; j0<ndet; ++j0 ){
        for( int j1=0; j1<ng; ++j1 ){
            S.Clear();
            S.z.resize( 2 );
            S.z[0] = j0 ;
            S.z[1] = j1 ;
            olf.VS.push_back(S);
        }
    }

    string lin;
    vector<double> dat_f, dat_i;
    int j1 = 0, nline=0;
    double x, y, tmeas, ttot=0;
    while( mystd::freadln(fd_f, &lin)>0 ){
        if( mystd::str2vec(lin.substr(1), &dat_f, 0, false) ) break;
        if( dat_f.size()!=ndet+2 ){
            cout << "! line "<<nline<<" has size "<<dat_f.size()<<
                " instead of "<<ndet+2<<"\n";
            return;
        }
        if( diff ){
            if( mystd::freadln(fd_i, &lin) <= 0 ){
                cout << "! incongruent initial file length\n";
                return;
            }
            if( mystd::str2vec(lin.substr(1), &dat_i, 0, false) ){
                cout << "! incongruent line in initial file\n";
                return;
            }
            if( dat_i.size()!=ndet+2 ){
                cout << "! initial file: line "<<nline<<" has size "
                     <<dat_i.size()<<" instead of "<<ndet+2<<"\n";
                return;
            }
        }                
        x = dat_f[0]+0.5;
        tmeas = dat_f[ndet+1];
        if( diff ){
            if( dat_i[0] != x ){
                cout<<"! inconsistent phase\n";
                return;
            }
            tmeas -= dat_i[ndet+1];
            if( tmeas<0 ){
                cout<<"! negative measuring time\n";
                return;
            }
        }
        // cout<<"measured "<<tmeas<<" times at "<<x<<"\n";
        if( tmeas>0 ){
            for ( int j0=0; j0<ndet; ++j0 ){
                y = dat_f[1+j0];
                if( diff ) y -= dat_i[1+j0];
                y = y/tmeas/2e-5;
                olf.VS[j0*ng+j1].push_back( x, y );
            }
            ttot += tmeas*2e-5;
        }
        j1 = (++j1)%ng;
        ++nline;
    }

    cout<<"effective acquisition time "<<ttot<<" sec\n";
    olf.RPar.push_back( CParam( "eff. acquisition time", "sec", ttot ) );
    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}

// ***   rss_stream output specifities    *******************************

namespace NR_Stream{
    const int NDET = 19;
    const int NREC = 50000;
    int GetVelocity( int ipos, int iline );
    static int maxsisdef = 0;
    static bool heal_dopp_jumps_def = true;
}

class CRStream{
public:
    inline CRStream(): maxsis(0) {};
    int ask();
    int open();
    void rewind();
    int getrec( short& idopp, short& ichop, unsigned int * cnt );
    int maxsis;
    int nsis;
    int nrec;
    string fname;
    string name;
    bool heal_dopp_jumps;
    string doc;
private:
    void reset(){ nsis=0; nrec=0; ijump=0; idoppold=0; };
    FILE *fd;
    short ijump;
    short idoppold;
    // local in getrec, declared here in the hope for a speed benefit:
    unsigned char buf[4*NR_Stream::NDET+2];
    int nread;
};

int CRStream::ask()
{
    name = wask( "Read data from stream (.strm)" );
    if( name=="" ) return -1;
    fname = name + ".strm";
    maxsis = iask( "maximum number of blocks to read (0: read all)",
                   NR_Stream::maxsisdef );
    NR_Stream::maxsisdef = maxsis;
    heal_dopp_jumps = bask( "try to correct dopp pos jumps",
                            NR_Stream::heal_dopp_jumps_def );
    NR_Stream::heal_dopp_jumps_def = heal_dopp_jumps;
    doc = ( maxsis==0 ? "All" : strg(maxsis) ) + " blocks from " + fname
        + ", " + ( heal_dopp_jumps ? "with" : "without" )
        + " doppler position correction";
    return 0;
}

int CRStream::open()
{
    if (!(fd = fopen(fname.c_str(), "r"))) {
        printf ("! cannot open file %s:\n", fname.c_str());
        perror( 0 );
        return -1;
    }
    reset();
    return 0;
}

void CRStream::rewind()
{
    ::rewind( fd );
    reset();    
}

int CRStream::getrec( short& idoppout, short& ichop, unsigned int * cnt )
{
    // 4. Datenformat von rss_stream
    // 
    // (Das ist das vom DSP erzeugte Format.)
    // Ein Datenpuffer enthaelt 1000002 Worte (zu 4 Byte).
    // Wort 0: Gesamtlaenge in Worten (1000002)
    // Wort 1: Interruptcounter zur Diagnose
    // Wort 2..21: Datenblock 0
    // [...]
    // Wort 999982..1000001: Datenblock 49999
    // 
    // Ein Datenblock enthaelt 20 Worte.
    // Wort 0: Wert aus Latch (d.h. Doppler- und Chopper-Position)
    // Wort 1: Scalerkanal 0
    // [...]
    // Wort 19: Scalerkanal 18

    if( nrec==0 ){
        if( maxsis && nsis==maxsis ) return -1; // user wanted to stop here

        if( (nread = fread( buf, 1, 8, fd ))!=8 ) return -2; // regular EOF

        printf( "%s %7i\n", name.c_str(), nsis ); // show progress
    }
    short idopp;
    if( fread( &idopp, 2, 1, fd )!=1 ) {
        printf( "failed reading doppler\n" );
        return -3;
    }
    
    if( fread( &ichop, 2, 1, fd )!=1 ){
        printf( "failed reading chopper\n" );
        return -4;
    }
    
    if( (nread = fread( cnt, 4, NR_Stream::NDET, fd ))!=NR_Stream::NDET ){
        printf( "failed reading counters (%i)\n", nread );
        return -5;
    }
    
    if( heal_dopp_jumps ){
        short idiff = idoppold-idopp;
        if( nsis && nrec && abs( idiff )>12 ){ 
            int ijumpold = ijump;
            ijump += (8+idiff)>>4<<4; // round to 2^4
        }
        idoppold = idopp;
        idoppout = idopp+ijump;
    } else {
        idoppout = idopp;
    }

    ++nrec;
    if( nrec==NR_Stream::NREC ){
        nrec = 0;
        ++nsis;
    }

    return 0;
}

void NR_Stream::Stream2DoppChop(void)
{
    COld olf;
    CScan S;
    FILE *fs0, *fs1;
        
    CRStream rs;
    if( rs.ask() ) return;
    if( rs.open() ) return;

    static bool save_here = true;
    save_here = bask( "save as online file", save_here );

    if( save_here ){
        olf.name = rs.name;
        olf.xco = CCoord( "chop", "" );
        olf.yco = CCoord( "dopp", "" );
        olf.lDoc.push_back( rs.doc );
    } else {
        string outFile = rs.name+".dopp";
        if (!(fs0 = fopen( outFile.c_str(), "w"))) {
            printf ("! cannot open file %s\n", outFile.c_str());
            return;
        }
        outFile = rs.name+".chop";
        if (!(fs1 = fopen( outFile.c_str(), "w"))) {
            printf ("! cannot open file %s\n", outFile.c_str());
            return;
        }
    }

    short idopp, ichop;
    uint cnt[NDET];

    while( !rs.getrec( idopp, ichop, cnt ) ){

        if( save_here ){
            S.push_back( (double)ichop, (double)idopp );
        } else {
            fprintf( fs0, "%6i\n", idopp );
            fprintf( fs1, "%6i\n", ichop );
        }
    }

    if( save_here ){
        olf.VS.push_back( S );
        NOlm::SelNew();
        NOlm::OloAdd(&olf);
    } else {
        fclose(fs0);
        fclose(fs1);
    }
}

void NR_Stream::ReadStream(void)
{
    COld olf;

    CRStream rs;
    if( rs.ask() ) return;
    if( rs.open() ) return;

    olf.lDoc.push_back( rs.doc );

    olf.xco = CCoord( "dopp", "" );
    olf.yco = CCoord( "counts or time", "" );
    olf.ZCo.push_back( CCoord( "det", "") );
    olf.ZCo.push_back( CCoord( "chop", "") );
    olf.name = rs.name;

    static int ni = 400;
    const int nz0 = NDET; // Detektoren
    static int nz1 = 60;
    const int ndopp = 40000;
    // nz1 = iask( "divide chopper into how many channels", nz1 );
    // ni = iask( "accumulate doppler into how many channels", ni );
    
    CScan S;
    // prepare empty data array:
    for ( int j0=-1; j0<nz0; ++j0 ) {
        for ( int j1=0; j1<nz1; ++j1 ){
            S.Clear();
            S.z.resize( 2 );
            S.z[0] = j0 ;
            S.z[1] = j1*120/nz1 ;
            for( int i=0; i<ni; ++i ){
                S.push_back( (double)i, 0.0 );
            }
            olf.VS.push_back(S);
        }
    }

    short idopp, ichop;
    uint cnt[NDET];

    while( !rs.getrec( idopp, ichop, cnt ) ){

        int iy = 20000+idopp; // GetVelocity( (int)idopp, nsis*NREC+nrec );
        if (iy < 0) continue;
        iy = iy*ni/ndopp;
        if( iy>=ni ){
            // iy = ni-1;
            printf( "sis %i record %i\n", rs.nsis, rs.nrec );
            printf( "unexpected i=%i (u %u) from idopp=%i (u %u) where ni=%i\n", iy, iy, idopp, idopp, ni );
            continue;
        }
        
        uint j1 = (ichop*nz1*3/20480) % nz1;
        if( j1<0 || j1>=nz1 ){
            printf( "sis %i record %i\n", rs.nsis, rs.nrec );
            printf( "unexpected j1=%i (u %u) from ichop=%i (u %u)\n", j1, j1, ichop, ichop );
            continue;
        }
        
        for( int j0=-1; j0<nz0; ++j0 ){
            olf.VS[(j0+1)*nz1+j1].y[iy] += j0<0 ? 2e-5 : cnt[j0];
        }
        
    }
    printf( "stream with %i times %i records covered %f seconds\n", rs.nsis, NREC, 2e-5*rs.nsis*NREC );

    NOlm::SelNew();
    NOlm::OloAdd(&olf);
}
